@@ -50,8 +50,8 @@
 #include <libgnomevfs/gnome-vfs-file-info.h>
 #include <libgnomevfs/gnome-vfs-ops.h>
 #ifdef HAS_OPENBABEL_2
-#	include <mol.h>
-#	include <obconversion.h>
+#	include <openbabel/mol.h>
+#	include <openbabel/obconversion.h>
 #else
 #	include <sstream>
 #endif
