@@ -105,6 +105,13 @@ bool gcpFragment::OnChanged(GtkTextBuffer *textbuffer)
 		gnome_canvas_rich_text_ext_get_iter_location(text, &iter, &rect);
 		m_height = rect.height + 1;
 		m_length = 1;
+		GtkTextMark *mark = gtk_text_buffer_get_mark (m_buf, "insert");
+		gtk_text_buffer_get_iter_at_mark(m_buf, &start, mark);
+		gtk_text_buffer_delete_mark (m_buf, m_BeginAtom);
+		m_BeginAtom = gtk_text_buffer_create_mark (m_buf, "ab0", &start, true);
+		gtk_text_buffer_delete_mark (m_buf, m_EndAtom);
+		m_EndAtom = gtk_text_buffer_create_mark (m_buf, "ae0", &start, false);
+		m_lbearing = 0;
 		pView->Update (this);
 		EmitSignal (OnChangedSignal);
 		m_bLoading = false;
@@ -149,24 +156,34 @@ bool gcpFragment::OnChanged(GtkTextBuffer *textbuffer)
 		//atom has been partly deleted at least
 			Z = GetElementAtIter(&start, &iter);
 			m_Atom->SetZ(Z);
-			if (!Z)
-			{
-				gtk_text_buffer_delete_mark(m_buf, m_BeginAtom);
-				m_BeginAtom = gtk_text_buffer_create_mark(m_buf, "ab0", &start, true);
+			if (!Z) {
+				GtkTextMark *mark = gtk_text_buffer_get_mark (m_buf, "insert");
+				gtk_text_buffer_get_iter_at_mark(m_buf, &start, mark);
+				gtk_text_buffer_delete_mark (m_buf, m_BeginAtom);
+				m_BeginAtom = gtk_text_buffer_create_mark (m_buf, "ab0", &start, false);
+				gtk_text_buffer_delete_mark (m_buf, m_EndAtom);
+				m_EndAtom = gtk_text_buffer_create_mark (m_buf, "ae0", &start, true);
+			} else {
+				gtk_text_buffer_delete_mark(m_buf, m_EndAtom);
+				m_EndAtom = gtk_text_buffer_create_mark(m_buf, "ae0", &iter, true);
 			}
-			gtk_text_buffer_delete_mark(m_buf, m_EndAtom);
-			m_EndAtom = gtk_text_buffer_create_mark(m_buf, "ae0", &iter, true);
 		}
 		else if (strlen(symbol) < 3)
 		{
-			Z = GetElementAtIter(&start, &iter);
-			if (Z)
-			{
-				m_Atom->SetZ(Z);
-				gtk_text_buffer_delete_mark(m_buf, m_BeginAtom);
-				m_BeginAtom = gtk_text_buffer_create_mark(m_buf, "ab0", &start, false);
-				gtk_text_buffer_delete_mark(m_buf, m_EndAtom);
-				m_EndAtom = gtk_text_buffer_create_mark(m_buf, "ae0", &iter, true);
+			Z = GetElementAtIter (&start, &iter);
+			if (Z) {
+				m_Atom->SetZ (Z);
+				gtk_text_buffer_delete_mark (m_buf, m_BeginAtom);
+				m_BeginAtom = gtk_text_buffer_create_mark (m_buf, "ab0", &start, false);
+				gtk_text_buffer_delete_mark (m_buf, m_EndAtom);
+				m_EndAtom = gtk_text_buffer_create_mark (m_buf, "ae0", &iter, true);
+			} else {
+				GtkTextMark *mark = gtk_text_buffer_get_mark (m_buf, "insert");
+				gtk_text_buffer_get_iter_at_mark(m_buf, &start, mark);
+				gtk_text_buffer_delete_mark (m_buf, m_BeginAtom);
+				m_BeginAtom = gtk_text_buffer_create_mark (m_buf, "ab0", &start, false);
+				gtk_text_buffer_delete_mark (m_buf, m_EndAtom);
+				m_EndAtom = gtk_text_buffer_create_mark (m_buf, "ae0", &start, true);
 			}
 		}
 	}
@@ -366,14 +383,14 @@ void gcpFragment::Add(GtkWidget* w)
 			y += ChargeSignHeight / 2.;
 			break;
 		case -1:
-			x -= ChargeSignSize -1.;
+			x -= ChargeSignSize + 2.;
 			break;
 		case 0:
 		case -3:
 			x -= ChargeSignSize / 2.;
 			break;
 		case 1:
-			x += ChargeSignSize + 1.;
+			x += 2.;
 			break;
 		case 2:
 			x -= ChargeSignSize / 2.;
@@ -381,7 +398,6 @@ void gcpFragment::Add(GtkWidget* w)
 			break;
 		}
 		y -= ChargeSignHeight / 2. - ChargeYAlign + ChargeSignSize - 1.;
-printf("x=%g y=%g\n",m_x,m_y);
 		chgp = (GnomeCanvasGroup*) gnome_canvas_item_new (
 					group,
 					gnome_canvas_group_ext_get_type(),
@@ -425,7 +441,6 @@ printf("x=%g y=%g\n",m_x,m_y);
 				);
 		gnome_canvas_path_def_unref (cpd);
 		g_object_set_data (G_OBJECT (group), "sign", item);
-puts("items added");
 	}
 	pData->Items[this] = group;
 	gpointer obj = g_object_get_data (G_OBJECT (group), "fragment");
@@ -547,14 +562,14 @@ void gcpFragment::Update(GtkWidget* w)
 				y += ChargeSignSize / 2.;
 				break;
 			case -1:
-				x -= ChargeSignSize -1.;
+				x -= ChargeSignSize + 2.;
 				break;
 			case 0:
 			case -3:
 				x -= ChargeSignSize / 2.;
 				break;
 			case 1:
-				x += ChargeSignSize + 1.;
+				x += 2.;
 				break;
 			case 2:
 				x -= ChargeSignSize / 2.;
@@ -1158,6 +1173,7 @@ int gcpFragment::GetChargePosition(gcpFragmentAtom* pAtom, unsigned char& Pos, d
 			return 0;
 	} else
 		return 0;
+
 	switch (Pos) {
 	case CHARGE_NE:
 		x = m_x + width / 2.0;
