@@ -44,8 +44,8 @@
 #include <unistd.h>
 #include <locale.h>
 #ifdef HAS_OPENBABEL_2
-#	include <mol.h>
-#	include <obconversion.h>
+#	include <openbabel/mol.h>
+#	include <openbabel/obconversion.h>
 #endif
 
 static void do_export_to_ghemical (gcpMolecule* pMol)
@@ -514,7 +514,7 @@ void gcpMolecule::ExportToGhemical ()
 #ifdef HAS_OPENBABEL_2
 	OBMol Mol;
 	OBConversion Conv;
-	OBFormat* pOutFormat = Conv.FormatFromExt ("gpr");
+	OBFormat* pOutFormat = Conv.FindFormat ("gpr");
 	Conv.SetInAndOutFormats (pOutFormat, pOutFormat);
 #else
 	OBMol Mol(UNDEFINED, GHEMICAL);
